@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: bytegen.ml,v 1.65.2.1 2004/07/07 16:49:51 xleroy Exp $ *)
+(* $Id: bytegen.ml,v 1.67 2005/08/25 15:35:16 doligez Exp $ *)
 
 (*  bytegen.ml : translation of lambda terms to lists of instructions. *)
 
@@ -146,7 +146,7 @@ let rec size_of_lambda = function
 let copy_event ev kind info repr =
   { ev_pos = 0;                   (* patched in emitcode *)
     ev_module = ev.ev_module;
-    ev_char = ev.ev_char;
+    ev_loc = ev.ev_loc;
     ev_kind = kind;
     ev_info = info;
     ev_typenv = ev.ev_typenv;
@@ -686,7 +686,7 @@ let rec comp_expr env exp sz cont =
       let event kind info =
         { ev_pos = 0;                   (* patched in emitcode *)
           ev_module = !compunit_name;
-          ev_char = lev.lev_pos;
+          ev_loc = lev.lev_loc;
           ev_kind = kind;
           ev_info = info;
           ev_typenv = lev.lev_env;
