@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: grammar.ml,v 1.12.2.2 2004/11/22 13:41:24 mauny Exp $ *)
+(* $Id: grammar.ml,v 1.14 2005/03/24 17:20:53 doligez Exp $ *)
 
 open Stdpp;
 open Gramext;
@@ -224,7 +224,7 @@ value rec name_of_symbol entry =
 
 value rec get_token_list entry tokl last_tok tree =
   match tree with
-  [ Node {node = (Stoken tok as s); son = son; brother = DeadEnd} ->
+  [ Node {node = Stoken tok; son = son; brother = DeadEnd} ->
       get_token_list entry [last_tok :: tokl] tok son
   | _ ->
       if tokl = [] then None
