@@ -11,20 +11,20 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: point_col.c,v 1.7 2001/12/07 13:39:55 xleroy Exp $ */
+/* $Id: point_col.c,v 1.8 2004/03/24 15:02:05 starynke Exp $ */
 
 #include "libgraph.h"
 
-value gr_point_color(value vx, value vy)
+value caml_gr_point_color(value vx, value vy)
 {
   int x = Int_val(vx);
   int y = Int_val(vy);
   XImage * im;
   int rgb;
 
-  gr_check_open();
-  im = XGetImage(grdisplay, grbstore.win, x, Bcvt(y), 1, 1, (-1), ZPixmap);
-  rgb = gr_rgb_pixel(XGetPixel(im, 0, 0));
+  caml_gr_check_open();
+  im = XGetImage(caml_gr_display, caml_gr_bstore.win, x, Bcvt(y), 1, 1, (-1), ZPixmap);
+  rgb = caml_gr_rgb_pixel(XGetPixel(im, 0, 0));
   XDestroyImage(im);
   return Val_int(rgb);
 }
