@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: topdirs.mli,v 1.10 2001/07/03 11:04:10 xleroy Exp $ *)
+(* $Id: topdirs.mli,v 1.12 2002/04/18 07:27:44 garrigue Exp $ *)
 
 (* The toplevel directives. *)
 
@@ -30,3 +30,5 @@ val dir_untrace_all : formatter -> unit -> unit
 type 'a printer_type_new = Format.formatter -> 'a -> unit
 type 'a printer_type_old = 'a -> unit
 
+(* For topmain.ml. Maybe shouldn't be there *)
+val load_file : formatter -> string -> bool
