@@ -10,12 +10,13 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: warnings.mli,v 1.13 2003/05/02 08:46:06 weis Exp $ *)
+(* $Id: warnings.mli,v 1.16 2005/09/15 03:09:26 garrigue Exp $ *)
 
 open Format
 
 type t =                             (* A is all *)
-  | Comment of string                (* C *)
+  | Comment_start                    (* C *)
+  | Comment_not_end
   | Deprecated                       (* D *)
   | Fragile_pat of string            (* E *)
   | Partial_application              (* F *)
@@ -24,9 +25,21 @@ type t =                             (* A is all *)
   | Partial_match of string          (* P *)
   | Statement_type                   (* S *)
   | Unused_match                     (* U *)
-  | Unused_pat                       (* U *)
+  | Unused_pat
   | Hide_instance_variable of string (* V *)
-  | Other of string                  (* X *)
+  | Illegal_backslash                (* X *)
+  | Implicit_public_methods of string list
+  | Unerasable_optional_argument
+  | Undeclared_virtual_method of string
+  | Not_principal of string
+  | Without_principality of string
+  | Unused_argument
+  | Nonreturning_statement
+  | Camlp4 of string
+  | All_clauses_guarded
+  | Useless_record_with
+  | Unused_var of string             (* Y *)
+  | Unused_var_strict of string      (* Z *)
 ;;
 
 val parse_options : bool -> string -> unit;;
